#define DEV_EC0_DATA_REG 0x62
#define DEV_EC0_CST_REG  0x66

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/io.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>


typedef unsigned short u16;
typedef unsigned char u8;

int done;

void sighandler() {
	done = 1;
}

void ecr_set16(u16 addr, u16 value) {
	while (inw(DEV_EC0_CST_REG) & 3) usleep(1);
	outw(0x81, DEV_EC0_CST_REG);
	while (inw(DEV_EC0_CST_REG) & 2) usleep(1);
	outw(addr, DEV_EC0_DATA_REG);
	while (inw(DEV_EC0_CST_REG) & 2) usleep(1);
	outw(value, DEV_EC0_DATA_REG);
	while (inw(DEV_EC0_CST_REG) & 2) usleep(1);
}

u16 ecr_get16(u16 addr) {
	while (inw(DEV_EC0_CST_REG) & 3) usleep(1);
	outb(0x80, DEV_EC0_CST_REG);
	while (inw(DEV_EC0_CST_REG) & 2) usleep(1);
	outb(addr, DEV_EC0_DATA_REG);
	while (inw(DEV_EC0_CST_REG) & 2) usleep(1);
	return inw(DEV_EC0_DATA_REG);
}

void ecr_set8(u8 addr, u8 value) {
	while (inb(DEV_EC0_CST_REG) & 3) usleep(1);
	outb(0x81, DEV_EC0_CST_REG);
	while (inb(DEV_EC0_CST_REG) & 2) usleep(1);
	outb(addr, DEV_EC0_DATA_REG);
	while (inb(DEV_EC0_CST_REG) & 2) usleep(1);
	outb(value, DEV_EC0_DATA_REG);
	while (inb(DEV_EC0_CST_REG) & 2) usleep(1);
}

u8 ecr_get8(u8 addr) {
	while (inb(DEV_EC0_CST_REG) & 3) usleep(1);
	outb(0x80, DEV_EC0_CST_REG);
	while (inb(DEV_EC0_CST_REG) & 2) usleep(1);
	outb(addr, DEV_EC0_DATA_REG);
	while (inb(DEV_EC0_CST_REG) & 2) usleep(1);
	return inb(DEV_EC0_DATA_REG);
}

int main (int argc, char **argv) {
	u8 brightness, battstat;
	u8 batt_bright, ac_bright;
	int ac;
    
	if (argc == 3) {
		ac_bright = atoi(argv[1]) & 0xFF;
		batt_bright = atoi(argv[2]) & 0xFF;
	} else {
		printf("Usage: %s ac_bright_val batt_bright_val\n", argv[0]);
		return 1;
	}

	done = 0;
	signal (SIGINT, sighandler);
	signal (SIGQUIT, sighandler);
	signal (SIGKILL, sighandler);
	signal (SIGTERM, sighandler);
	signal (SIGSTOP, sighandler);
	signal (SIGTSTP, sighandler);
	signal (SIGPWR, sighandler);
	iopl(3);
	/* first read the state and set brightness accordingly */
	battstat = ecr_get8(0x81);
	brightness = ecr_get8(0x96);
	ac = battstat & 0x04;
	if (ac) {
		/* printf("AC, Brightness %d/255\n", brightness); */
		ecr_set8(0x96, ac_bright);
	} else {
		/* printf("BATT, Brightness %d/255\n", brightness); */
		ecr_set8(0x96, batt_bright);
	}
    
	while(!done) {
		battstat = ecr_get8(0x81);
		brightness = ecr_get8(0x96);
		if (((battstat & 0x04) != ac) || 
		    (ac && (brightness != ac_bright)) ||
		    (!ac && (brightness != batt_bright))) {
			ac = battstat & 0x04;
			if (ac) {
				ecr_set8(0x96, ac_bright);
			} else {
				ecr_set8(0x96, batt_bright);
			}
		}
		usleep(1000);
	}
	return 0;
} 
